@@ -4,7 +4,7 @@ import re
 import os.path
 import logging
 
-from typing import List, Optional
+from typing import List, Optional, Callable
 
 LOG = logging.getLogger(__name__)
 
@@ -19,21 +19,112 @@ OSX_PARENTHESIZED_PROC = re.compile("^\\([^()]+\\)$")
 PERL_BIN = re.compile("^perl[.0-9]*$")
 
 
-def to_array(commandline: str) -> List[str]:
+def get_trailing_absolute_path(so_far: str) -> Optional[str]:
+    """
+    Extract a potential file path from the end of a string.
+
+    The coalescing logic will then base decisions on whether this file path
+    exists or not.
+    """
+    start_index = -1
+    if so_far.startswith("/"):
+        start_index = 0
+    elif so_far.startswith("-"):
+        equals_slash_index = so_far.find("=/")
+        if equals_slash_index == -1:
+            # No =/ in the string, so we're not looking at -Djava.io.tmpdir=/tmp
+            return None
+
+        # Start at the slash after the equals sign
+        start_index = equals_slash_index + 1
+
+    if start_index == -1:
+        # Start not found, this is not a path
+        return None
+
+    colon_slash_index = so_far.rfind(":/", start_index)
+    if colon_slash_index == -1:
+        # Not a : separated path
+        return so_far[start_index:]
+
+    return so_far[colon_slash_index + 1 :]
+
+
+def should_coalesce(
+    parts: List[str], exists: Callable[[str], bool] = os.path.exists
+) -> Optional[bool]:
+    """
+    Two or more (previously) space separated command line parts should be
+    coalesced if combining them with a space in between creates an existing file
+    path, or a : separated series of file paths.
+
+    Return values:
+    * True: Coalesce, done
+    * False: Do not coalesce, done
+    * None: Undecided, add another part and try again
+    """
+
+    if parts[-1].startswith("-") or parts[-1].startswith("/"):
+        # Last part starts a command line option or a new absolute path, don't
+        # coalesce.
+        return False
+
+    coalesced = " ".join(parts)
+    candidate = get_trailing_absolute_path(coalesced)
+    if not candidate:
+        # This is not a candidate for coalescing
+        return False
+
+    if exists(candidate):
+        # Found it, done!
+        return True
+
+    if exists(os.path.dirname(candidate)):
+        # Found the parent directory, we're on the right track, keep looking!
+        return None
+
+    # Candidate does not exists, and neither does its parent directory, this is
+    # not it.
+    return False
+
+
+def coalesce_count(
+    parts: List[str], exists: Callable[[str], bool] = os.path.exists
+) -> int:
+    """How many parts should be coalesced?"""
+
+    for coalesce_count in range(2, len(parts) + 1):
+        should_coalesce_ = should_coalesce(parts[0:coalesce_count], exists)
+
+        if should_coalesce_ is None:
+            # Undecided, keep looking
+            continue
+
+        if should_coalesce_ is False:
+            return 1
+
+        # should_coalesce_ is True
+        return coalesce_count
+
+    # Undecided until the end, this means no coalescing should be done
+    return 1
+
+
+def to_array(
+    commandline: str, exists: Callable[[str], bool] = os.path.exists
+) -> List[str]:
     """Splits a command line string into components"""
     base_split = commandline.split(" ")
     if len(base_split) == 1:
         return base_split
 
-    # Try to reverse engineer executables with spaces in their names
-    merged_split = list(base_split)
-    while not os.path.isfile(merged_split[0]):
-        if len(merged_split) == 1:
-            # Nothing more to merge, give up
-            return base_split
-
-        # Merge the two first elements: http://stackoverflow.com/a/1142879/473672
-        merged_split[0:2] = [" ".join(merged_split[0:2])]
+    # Try to reverse engineer which spaces should not be split on
+    merged_split: List[str] = []
+    i = 0
+    while i < len(base_split):
+        coalesce_count_ = coalesce_count(base_split[i:], exists)
+        merged_split.append(" ".join(base_split[i : i + coalesce_count_]))
+        i += coalesce_count_
 
     return merged_split
 
@@ -121,6 +212,9 @@ def get_command(commandline: str) -> str:
     if command == "java":
         return faillog(commandline, get_java_command(commandline))
 
+    if command == "dart":
+        return faillog(commandline, get_dart_command(commandline))
+
     if command == "ruby":
         # Switches list inspired by ruby 2.3.7p456 --help output
         return faillog(
@@ -190,8 +284,11 @@ def get_command(commandline: str) -> str:
             ),
         )
 
-    if command in ["bash", "sh"]:
-        return faillog(commandline, get_generic_script_command(commandline))
+    if command in ["bash", "sh", "zsh"]:
+        return faillog(
+            commandline,
+            get_generic_script_command(commandline, ignore_switches=["-p"]),
+        )
 
     if PERL_BIN.match(command):
         return faillog(commandline, get_generic_script_command(commandline))
@@ -255,13 +352,64 @@ def get_python_command(commandline: str) -> Optional[str]:
     if len(array) == 1:
         return python
 
-    if not array[1].startswith("-"):
-        return os.path.basename(array[1])
-
     if len(array) > 2:
         if array[1] == "-m" and not array[2].startswith("-"):
             return os.path.basename(array[2])
 
+    if array[1].startswith("-"):
+        return None
+
+    if os.path.basename(array[1]) == "aws":
+        # Drop the "python" part of "python aws"
+        return get_aws_command(array[1:])
+
+    return os.path.basename(array[1])
+
+
+def get_aws_command(args: List[str]) -> Optional[str]:
+    '''Extract "aws command subcommand" from a command line starting with "aws"'''
+    result = ["aws"]
+    for arg in args[1:]:
+        if arg.startswith("--profile="):
+            continue
+        if arg.startswith("--region="):
+            continue
+        if arg.startswith("-"):
+            break
+        if os.path.sep in arg:
+            break
+
+        result.append(arg)
+        if len(result) >= 4:
+            # Got "aws command subcommand"
+            break
+
+    if len(result) == 4 and result[-1] != "help":
+        del result[-1]
+
+    return " ".join(result)
+
+
+def get_dart_command(commandline: str) -> Optional[str]:
+    """Returns None if we failed to figure out the script name"""
+    array = to_array(commandline)
+    dart = os.path.basename(array[0])
+    if len(array) == 1:
+        return dart
+
+    for candidate in array[1:]:
+        if candidate.startswith("-"):
+            continue
+
+        for _, char in enumerate(candidate):
+            if not char.islower():
+                # Not a subcommand, something like "/bin/hello.dart"
+                return os.path.basename(candidate)
+
+        # This was a subcommand
+        return dart + " " + candidate
+
+    # Neither subcommand nor file name found, give up
     return None
 
 
@@ -389,7 +537,7 @@ def get_with_subcommand(
 
     if array[1].startswith("-"):
         # Unknown option, help!
-        return command
+        return None
 
     return f"{command} {array[1]}"
 
@@ -417,9 +565,8 @@ def get_generic_script_command(
     if len(array) == 2:
         # vm + script
         return script
-    if script not in ["brew.rb", "yarn.js"]:
+    if script not in ["brew.rb", "brew.sh", "yarn.js"]:
         return script
-    script = os.path.splitext(script)[0]
 
     subcommand = array[2]
     if subcommand.startswith("-"):
