@@ -7,14 +7,12 @@ import re
 import pwd
 import errno
 import subprocess
-import dateutil.tz
 
 from . import px_commandline
 from . import px_exec_util
 
 
 from typing import Dict
-from typing import MutableSet
 from typing import Optional
 from typing import List
 from typing import Iterable
@@ -38,7 +36,7 @@ CPUTIME_LINUX = re.compile("^([0-9][0-9]):([0-9][0-9]):([0-9][0-9])$")
 CPUTIME_LINUX_DAYS = re.compile("^([0-9]+)-([0-9][0-9]):([0-9][0-9]):([0-9][0-9])$")
 
 
-TIMEZONE = dateutil.tz.tzlocal()
+TIMEZONE = datetime.datetime.now(datetime.timezone.utc).astimezone().tzinfo
 
 
 uid_to_username_cache: Dict[int, str] = {}
@@ -81,7 +79,6 @@ def _parse_time(time_s: str) -> datetime.datetime:
 
 
 class PxProcess:
-    # pylint: disable=attribute-defined-outside-init
     def __init__(
         self,
         cmdline: str,
@@ -94,6 +91,7 @@ class PxProcess:
         memory_percent: Optional[float] = None,
         cpu_percent: Optional[float] = None,
         cpu_time: Optional[float] = None,
+        aggregated_cpu_time: float = 0.0,
     ) -> None:
         self.pid: int = pid
         self.ppid: Optional[int] = ppid
@@ -105,27 +103,6 @@ class PxProcess:
 
         self.start_time = _parse_time(start_time_string.strip())
         self.age_seconds: float = (now - self.start_time).total_seconds()
-        if self.age_seconds < -10:
-            # See: https://github.com/walles/px/issues/84
-            #
-            # We used to check for negative age, but since we look at the clock
-            # once to begin with, and then spend some milliseconds calling ps,
-            # we can sometimes find new processes with a timestamp that is newer
-            # than "now".
-            #
-            # If this is the cause, we should be well below 10s, since process
-            # listing doesn't take that long.
-            #
-            # If it takes more than 10s, something else is likely up.
-            LOG.error(
-                "Process age < -10: age_seconds=%r now=%r start_time=%r start_time_string=%r timezone=%r",
-                self.age_seconds,
-                now,
-                self.start_time,
-                start_time_string.strip(),
-                datetime.datetime.now(TIMEZONE).tzname(),
-            )
-            assert False
         if self.age_seconds < 0:
             self.age_seconds = 0
 
@@ -143,12 +120,16 @@ class PxProcess:
         if cpu_percent is not None:
             self.cpu_percent_s = f"{cpu_percent:.0f}%"
 
-        # Setting the CPU time like this implicitly recomputes the score
         self.set_cpu_time_seconds(cpu_time)
+        self.set_aggregated_cpu_time_seconds(aggregated_cpu_time)
 
-        self.children: MutableSet[PxProcess] = set()
+        self.children: List[PxProcess] = []
         self.parent: Optional[PxProcess] = None
 
+        # How many levels down the tree this process is. Kernel is level 0, init
+        # level 1 and everything else 2 and up.
+        self.level = 0
+
     def __repr__(self):
         # I guess this is really what __str__ should be doing, but the point of
         # implementing this method is to make the py.test output more readable,
@@ -169,19 +150,6 @@ class PxProcess:
     def __hash__(self):
         return self.pid
 
-    def _recompute_score(self):
-        self.score = 0.0
-        if self.memory_percent is None:
-            return
-        if self.cpu_time_seconds is None:
-            return
-
-        self.score = (
-            (self.cpu_time_seconds + 1.0)
-            * (self.memory_percent + 1.0)
-            / (self.age_seconds + 1.0)
-        )
-
     def set_cpu_time_seconds(self, seconds: Optional[float]) -> None:
         self.cpu_time_s: str = "--"
         self.cpu_time_seconds = None
@@ -189,7 +157,9 @@ class PxProcess:
             self.cpu_time_s = seconds_to_str(seconds)
             self.cpu_time_seconds = seconds
 
-        self._recompute_score()
+    def set_aggregated_cpu_time_seconds(self, seconds: float) -> None:
+        self.aggregated_cpu_time_s = seconds_to_str(seconds)
+        self.aggregated_cpu_time_seconds = seconds
 
     def match(self, string, require_exact_user=True):
         """
@@ -198,7 +168,7 @@ class PxProcess:
         See px_process_test.test_match() for the exact definition of how the
         matching is done.
         """
-        if string is None:
+        if not string:
             return True
 
         if self.username == string:
@@ -270,7 +240,6 @@ class PxProcessBuilder:
         self.memory_percent: Optional[float] = None
 
     def __repr__(self):
-        # pylint: disable=consider-using-f-string
         return (
             "start_time_string=%r pid=%r ppid=%r user=%r cpu%%=%r cputime=%r mem%%=%r cmd=<%r>"
             % (
@@ -374,9 +343,9 @@ def create_kernel_process(now: datetime.datetime) -> PxProcess:
     process_builder.ppid = None
 
     # FIXME: This should be the system boot timestamp, not the epoch
-    process_builder.start_time_string = datetime.datetime.utcfromtimestamp(0).strftime(
-        "%c"
-    )
+    process_builder.start_time_string = datetime.datetime.fromtimestamp(
+        0, datetime.timezone.utc
+    ).strftime("%c")
 
     process_builder.rss_kb = 0
     process_builder.username = "root"
@@ -412,7 +381,7 @@ def resolve_links(processes: Dict[int, PxProcess], now: datetime.datetime) -> No
             process.parent = processes.get(process.ppid)
 
         if process.parent is not None:
-            process.parent.children.add(process)
+            process.parent.children.append(process)
 
 
 def remove_process_and_descendants(processes: Dict[int, PxProcess], pid: int) -> None:
@@ -458,7 +427,6 @@ def get_all() -> List[PxProcess]:
             close_fds=close_fds,
             env=px_exec_util.ENV,
         ) as ps:
-
             stdout = ps.stdout
             assert stdout
             now = datetime.datetime.now().replace(tzinfo=TIMEZONE)
@@ -476,15 +444,75 @@ def get_all() -> List[PxProcess]:
 
 
 def order_best_last(processes: Iterable[PxProcess]) -> List[PxProcess]:
-    """Returns process list ordered with the most interesting one last"""
-    return sorted(processes, key=operator.attrgetter("score", "cmdline"))
+    """
+    Returns process list ordered with the most interesting one last.
+
+    The interestingness of a process is determined by three factors:
+    * How recently it was started. More recently is more interesting.
+    * How much memory it is using. Higher memory usage is more interesting.
+    * How much CPU it is using right now. Higher CPU usage is more interesting.
+    * How much CPU time it has used. Higher CPU time is more interesting.
+
+    To sort by all three of these factors, we first make three lists. The score
+    of each process is determined by its best position in any of the lists.
+    """
+
+    # Earlier in these lists = more interesting
+    by_age = sorted(
+        filter(lambda p: p.age_seconds is not None, processes),
+        key=operator.attrgetter("age_seconds"),
+    )
+    by_memory = sorted(
+        filter(lambda p: p.memory_percent is not None, processes),
+        key=operator.attrgetter("memory_percent"),
+        reverse=True,
+    )
+    by_cpu = sorted(
+        filter(lambda p: p.cpu_percent is not None, processes),
+        key=operator.attrgetter("cpu_percent"),
+        reverse=True,
+    )
+    by_cpu_time = sorted(
+        filter(lambda p: p.cpu_time_seconds is not None, processes),
+        key=operator.attrgetter("cpu_time_seconds"),
+        reverse=True,
+    )
+
+    age_to_position: Dict[int, int] = {}
+    for position, process in enumerate(by_age):
+        age_to_position[process.pid] = position
+
+    memory_to_position: Dict[float, int] = {}
+    for position, process in enumerate(by_memory):
+        memory_to_position[process.pid] = position
+
+    cpu_to_position: Dict[float, int] = {}
+    for position, process in enumerate(by_cpu):
+        cpu_to_position[process.pid] = position
+
+    cpu_time_to_position: Dict[float, int] = {}
+    for position, process in enumerate(by_cpu_time):
+        cpu_time_to_position[process.pid] = position
+
+    # Presumably higher than the number of processes on any system
+    NOT_FOUND = 99999
+
+    def score(process: PxProcess) -> int:
+        """Higher score = more interesting"""
+        best_position = min(
+            age_to_position.get(process.pid, NOT_FOUND),
+            memory_to_position.get(process.pid, NOT_FOUND),
+            cpu_to_position.get(process.pid, NOT_FOUND),
+            cpu_time_to_position.get(process.pid, NOT_FOUND),
+        )
+        return -best_position
+
+    return sorted(processes, key=score)
 
 
 def order_best_first(processes: Iterable[PxProcess]) -> List[PxProcess]:
     """Returns process list ordered with the most interesting one first"""
-    ordered = sorted(processes, key=operator.attrgetter("cmdline"))
-    ordered = sorted(ordered, key=operator.attrgetter("score"), reverse=True)
-    return ordered
+    return order_best_last(processes)[::-1]
 
 
 def seconds_to_str(seconds: float) -> str:
