@@ -3,6 +3,161 @@ import os
 from px import px_commandline
 
 
+def test_get_coalesce_candidate():
+    assert px_commandline.get_trailing_absolute_path("/hello") == "/hello"
+    assert px_commandline.get_trailing_absolute_path("/hello:/baloo") == "/baloo"
+
+    assert px_commandline.get_trailing_absolute_path("-Dx=/hello") == "/hello"
+    assert px_commandline.get_trailing_absolute_path("-Dx=/hello:/baloo") == "/baloo"
+
+    assert px_commandline.get_trailing_absolute_path("hello") is None
+    assert px_commandline.get_trailing_absolute_path("hello:/baloo") is None
+
+    assert (
+        px_commandline.get_trailing_absolute_path(
+            "/A/IntelliJ IDEA.app/C/p/mm/lib/mm.jar:/A/IntelliJ IDEA.app/C/p/ms/lib/ms.jar:/A/IntelliJ"
+        )
+        == "/A/IntelliJ"
+    )
+
+
+def test_coalesce_count():
+    def exists(s):
+        return s in ["/", "/a b c", "/a b c/"]
+
+    assert px_commandline.coalesce_count(["/a", "b", "c"], exists=exists) == 3
+    assert px_commandline.coalesce_count(["/a", "b", "c/"], exists=exists) == 3
+    assert px_commandline.coalesce_count(["/a", "b", "c", "d"], exists=exists) == 3
+
+    assert (
+        px_commandline.coalesce_count(["/a", "b", "c:/a", "b", "c"], exists=exists) == 5
+    )
+    assert (
+        px_commandline.coalesce_count(["/a", "b", "c/:/a", "b", "c/"], exists=exists)
+        == 5
+    )
+
+    assert (
+        px_commandline.coalesce_count(["/a", "b", "c:/a", "b", "c", "d"], exists=exists)
+        == 5
+    )
+    assert (
+        px_commandline.coalesce_count(
+            ["/a", "b", "c/:/a", "b", "c/", "d/"], exists=exists
+        )
+        == 5
+    )
+
+
+def test_to_array_spaced1():
+    assert px_commandline.to_array(
+        "java -Dhello=/Applications/IntelliJ IDEA.app/Contents",
+        exists=lambda s: s
+        in [
+            "/Applications",
+            "/Applications/IntelliJ IDEA.app",
+            "/Applications/IntelliJ IDEA.app/Contents",
+        ],
+    ) == ["java", "-Dhello=/Applications/IntelliJ IDEA.app/Contents"]
+
+
+def test_to_array_spaced2():
+    assert px_commandline.to_array(
+        " ".join(
+            [
+                "java",
+                "-Dhello=/Applications/IntelliJ IDEA.app/Contents/Info.plist",
+                "-classpath",
+                "/Applications/IntelliJ",
+                "IDEA.app/Contents/plugins/maven-model/lib/maven-model.jar:/Applications/IntelliJ",
+                "IDEA.app/Contents/plugins/maven-server/lib/maven-server.jar:/Applications/IntelliJ",
+                "IDEA.app/Contents/plugins/maven/lib/maven3-server-common.jar",
+                "MainClass",
+            ]
+        ),
+        exists=lambda s: s
+        in [
+            "/Applications",
+            "/Applications/IntelliJ IDEA.app/Contents/Info.plist",
+            "/Applications/IntelliJ IDEA.app/Contents/plugins/maven-model/lib/maven-model.jar",
+            "/Applications/IntelliJ IDEA.app/Contents/plugins/maven-server/lib/maven-server.jar",
+            "/Applications/IntelliJ IDEA.app/Contents/plugins/maven/lib/maven3-server-common.jar",
+        ],
+    ) == [
+        "java",
+        "-Dhello=/Applications/IntelliJ IDEA.app/Contents/Info.plist",
+        "-classpath",
+        "/Applications/IntelliJ IDEA.app/Contents/plugins/maven-model/lib/maven-model.jar:/Applications/IntelliJ IDEA.app/Contents/plugins/maven-server/lib/maven-server.jar:/Applications/IntelliJ IDEA.app/Contents/plugins/maven/lib/maven3-server-common.jar",
+        "MainClass",
+    ]
+
+
+def test_to_array_spaced3():
+    """Same as test_to_array_spaced2() but with two spaces rather than just one."""
+    assert px_commandline.to_array(
+        " ".join(
+            [
+                "java",
+                "-Dhello=/Applications/IntelliJ IDEA CE.app/Contents/Info.plist",
+                "-classpath",
+                "/Applications/IntelliJ",
+                "IDEA",
+                "CE.app/Contents/plugins/maven-model/lib/maven-model.jar:/Applications/IntelliJ",
+                "IDEA",
+                "CE.app/Contents/plugins/maven-server/lib/maven-server.jar:/Applications/IntelliJ",
+                "IDEA",
+                "CE.app/Contents/plugins/maven/lib/maven3-server-common.jar",
+                "MainClass",
+            ]
+        ),
+        exists=lambda s: s
+        in [
+            "/Applications",
+            "/Applications/IntelliJ IDEA CE.app/Contents/Info.plist",
+            "/Applications/IntelliJ IDEA CE.app/Contents/plugins/maven-model/lib/maven-model.jar",
+            "/Applications/IntelliJ IDEA CE.app/Contents/plugins/maven-server/lib/maven-server.jar",
+            "/Applications/IntelliJ IDEA CE.app/Contents/plugins/maven/lib/maven3-server-common.jar",
+        ],
+    ) == [
+        "java",
+        "-Dhello=/Applications/IntelliJ IDEA CE.app/Contents/Info.plist",
+        "-classpath",
+        "/Applications/IntelliJ IDEA CE.app/Contents/plugins/maven-model/lib/maven-model.jar:/Applications/IntelliJ IDEA CE.app/Contents/plugins/maven-server/lib/maven-server.jar:/Applications/IntelliJ IDEA CE.app/Contents/plugins/maven/lib/maven3-server-common.jar",
+        "MainClass",
+    ]
+
+
+def test_to_array_ms_edge():
+    complete = "/".join(
+        [
+            "/Applications",
+            "Microsoft Edge.app",
+            "Contents",
+            "Frameworks",
+            "Microsoft Edge Framework.framework",
+            "Versions",
+            "122.0.2365.63",
+            "Helpers",
+            "Microsoft Edge Helper (GPU).app",
+            "Contents",
+            "MacOS",
+            "Microsoft Edge Helper (GPU)",
+        ]
+    )
+    exists = []
+    partial = complete
+    while True:
+        exists.append(partial)
+        partial = os.path.dirname(partial)
+        if partial == "/":
+            break
+
+    assert px_commandline.to_array(
+        complete + " --type=gpu-process",
+        exists=lambda s: s in exists,
+    ) == [complete] + ["--type=gpu-process"]
+
+
 def test_get_command_python():
     assert px_commandline.get_command("python") == "python"
     assert px_commandline.get_command("/apa/Python") == "Python"
@@ -36,6 +191,40 @@ def test_get_command_python():
     assert px_commandline.get_command("python    ") == "python"
 
 
+def test_get_command_aws():
+    assert px_commandline.get_command("Python /usr/local/bin/aws") == "aws"
+    assert px_commandline.get_command("python aws s3") == "aws s3"
+    assert px_commandline.get_command("python3 aws s3 help") == "aws s3 help"
+    assert (
+        px_commandline.get_command("/wherever/python3 aws s3 help flaska")
+        == "aws s3 help"
+    )
+
+    assert px_commandline.get_command("python aws s3 sync help") == "aws s3 sync help"
+    assert px_commandline.get_command("python aws s3 sync nothelp") == "aws s3 sync"
+
+    assert px_commandline.get_command("python aws s3 --unknown sync") == "aws s3"
+
+    assert (
+        px_commandline.get_command(
+            " ".join(
+                [
+                    "python3",
+                    "/usr/local/bin/aws",
+                    "--profile=system-admin-prod",
+                    "--region=eu-west-1",
+                    "s3",
+                    "sync",
+                    "--only-show-errors",
+                    "s3://xxxxxx",
+                    "./xxxxxx",
+                ]
+            )
+        )
+        == "aws s3 sync"
+    )
+
+
 def test_get_command_java():
     assert px_commandline.get_command("java") == "java"
     assert px_commandline.get_command("java -version") == "java"
@@ -64,9 +253,6 @@ def test_get_command_java():
     assert px_commandline.get_command("java -jar") == "java"
     assert px_commandline.get_command("java -jar    ") == "java"
 
-    # FIXME: Add test for classpath containing spaces? I say this should be
-    # postponed until we have a real world use case for that.
-
 
 def test_get_command_java_gradled():
     commandline = (
@@ -179,6 +365,47 @@ def test_get_command_java_equinox():
     )
 
 
+# Ref:https://github.com/walles/px/issues/126
+def test_get_command_flutter():
+    # Candidate contains no slashes => subcommand
+    assert (
+        px_commandline.get_command(
+            "/usr/local/Caskroom/flutter/3.0.1/flutter/bin/cache/dart-sdk/bin/dart "
+            "devtools "
+            "--machine "
+            "--allow-embedding "
+        )
+        == "dart devtools"
+    )
+
+    # Candidate contains slashes => file to run
+    assert (
+        px_commandline.get_command(
+            "/usr/local/Caskroom/flutter/3.0.1/flutter/bin/cache/dart-sdk/bin/dart "
+            "--disable-dart-dev "
+            "--packages=/usr/local/Caskroom/flutter/3.0.1/flutter/packages/flutter_tools/.dart_tool/package_config.json "
+            "/usr/local/Caskroom/flutter/3.0.1/flutter/bin/cache/flutter_tools "
+            "upgrade"
+        )
+        == "flutter_tools"
+    )
+
+    # Candidate contains dots (.) => file to run
+    assert (
+        px_commandline.get_command(
+            "/usr/local/Caskroom/flutter/3.0.1/flutter/bin/cache/dart-sdk/bin/dart "
+            "--verbosity=error "
+            "--disable-dart-dev "
+            "--snapshot=/usr/local/Caskroom/flutter/3.0.1/flutter/bin/cache/flutter_tools.snapshot "
+            "--snapshot-kind=app-jit "
+            "--packages=/usr/local/Caskroom/flutter/3.0.1/flutter/packages/flutter_tools/.dart_tool/package_config.json "
+            "--no-enable-mirrors "
+            "flutter_tools.dart "
+        )
+        == "flutter_tools.dart"
+    )
+
+
 def test_get_command_electron_macos():
     # Note that if we have spaces inside of the path, the path in
     # question needs to be valid on the local system for it to
@@ -276,7 +503,7 @@ def test_get_command_ruby_switches():
         px_commandline.get_command(
             "/usr/bin/ruby -W0 /usr/local/bin/brew.rb install rust"
         )
-        == "brew install"
+        == "brew.rb install"
     )
 
     # https://github.com/walles/px/issues/87
@@ -394,7 +621,16 @@ def test_get_homebrew_commandline():
                 ]
             )
         )
-        == "brew upgrade"
+        == "brew.rb upgrade"
+    )
+
+
+def test_get_bash_brew_sh_commandline():
+    assert (
+        px_commandline.get_command(
+            "/bin/bash -p /usr/local/Homebrew/Library/Homebrew/brew.sh upgrade"
+        )
+        == "brew.sh upgrade"
     )
 
 
